#include <string>
#include "signals.h"

namespace signals
{
  std::string resize = "__RESIZE__";
  std::string quit = "__QUIT__";
  std::string highlight = "__HIGHLIGHT__";
  std::string highlightFirst = "__HIGHLIGHTFIRST__";
  std::string highlightLast = "__HIGHLIGHTLAST__";
  std::string highlightPrev = "__HIGHLIGHTPREV__";
  std::string highlightNext = "__HIGHLIGHTNEXT__";
  std::string highlightPrevPage = "__HIGHLIGHTPREVPAGE__";
  std::string highlightNextPage = "__HIGHLIGHTNEXTPAGE__";
  std::string keyTab = "__TAB__";
  std::string keyEnter = "__ENTER__";
  std::string scroll = "__SCROLL__";
  std::string keyRight = "__KEYRIGHT__";
  std::string keyLeft = "__KEYLEFT__";
  std::string keySpace = "__KEYSPACE__";
  std::string mouseEvent = "__MOUSEEVENT__";
  std::string nullEvent = "__NULLEVENT__";
  std::string tag = "__TAG__";
  std::string keyF9 = "__KEYF9__";
}
