#pragma once

#include <string>

namespace signals 
{
  extern std::string resize;
  extern std::string quit;
  extern std::string highlight;
  extern std::string highlightFirst;
  extern std::string highlightLast;
  extern std::string highlightPrev;
  extern std::string highlightNext;
  extern std::string highlightPrevPage;
  extern std::string highlightNextPage;
  extern std::string keyTab;
  extern std::string keyEnter;
  extern std::string scroll;
  extern std::string keyRight;
  extern std::string keyLeft;
  extern std::string keySpace;
  extern std::string mouseEvent;
  extern std::string nullEvent;
  extern std::string tag;
  extern std::string keyF9;
}
